package com.warewms.service.impl;

import com.alibaba.fastjson.JSON;
import com.jfinal.kit.Kv;
import com.jfinal.plugin.activerecord.Db;
import com.jfinal.plugin.activerecord.Record;
import com.jfinal.plugin.activerecord.SqlPara;
import com.warewms.common.annotation.JFinalTx;
import com.warewms.common.exception.RRException;
import com.warewms.common.utils.RecordUtils;
import com.warewms.entity.erp.*;
import com.warewms.service.ErpSyncService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class ErpSyncServiceImpl implements ErpSyncService {

    @Autowired
    private StringRedisTemplate redisTemplate;

    @JFinalTx
    @Override
    public void erpSync() {

        // 品号基础表 ERP_PRO_OB
        SqlPara sqlParaOb = Db.getSqlPara("erp.selectProOb");
        List<Record> recordObList = Db.find(sqlParaOb);
        List<ErpProOb> erpProObList = RecordUtils.converModel(recordObList, ErpProOb.class);
        //根据KEY 获取对应的 redis存储的value，存储数据为空进行新的存储
        String getErpProOb = redisTemplate.opsForValue().get("erpProOb");

        if (erpProObList != null && erpProObList.size() > 0 && StringUtils.isEmpty(getErpProOb)) {
            for (ErpProOb erpProOb : erpProObList) {
                Kv param = Kv.create();
                param.put("ima01", erpProOb.getIma01());
                SqlPara update = Db.getSqlPara("erp.updateProOb", param);
                Db.update(update);
            }
            redisTemplate.opsForValue().set("erpProOb", JSON.toJSONString(erpProObList));
        }

        // 产品类型基础表 ERP_PRO_TYPE
        SqlPara sqlPara = Db.getSqlPara("erp.selectProType");
        List<Record> recordList = Db.find(sqlPara);
        //根据KEY 获取对应的 redis存储的value，存储数据为空进行新的存储
        String getErpProType = redisTemplate.opsForValue().get("erpProType");

        if (recordList != null && recordList.size() > 0 && StringUtils.isEmpty(getErpProType)) {
            List<ErpProType> list = new ArrayList<>();
            for (Record record : recordList) {
                ErpProType proType = new ErpProType();
                proType.setTypeId(record.getStr("P_TYPE_ID"));
                proType.setTypeDesc(record.getStr("P_TYPE_DESC"));
                list.add(proType);

                Kv param = Kv.create();
                param.put("typeId", proType.getTypeId());
                SqlPara update = Db.getSqlPara("erp.updateProType", param);
                Db.update(update);
            }
            redisTemplate.opsForValue().set("erpProType", JSON.toJSONString(list));
        }

        // 条码品号对照表 ERP_PRO_BARCODE
        SqlPara sqlParaBarCode = Db.getSqlPara("erp.selectBarCode");
        List<Record> recordBarCodeList = Db.find(sqlParaBarCode);
        List<ErpProBarCode> erpBarCodeList = RecordUtils.converModel(recordBarCodeList, ErpProBarCode.class);
        //根据KEY 获取对应的 redis存储的value，存储数据为空进行新的存储
        String getErpBarCode = redisTemplate.opsForValue().get("erpBarCode");

        if (erpBarCodeList != null && erpBarCodeList.size() > 0 && StringUtils.isEmpty(getErpBarCode)) {
            for (ErpProBarCode erpProBarCode : erpBarCodeList) {
                Kv param = Kv.create();
                param.put("barcode", erpProBarCode.getBarcode());
                SqlPara update = Db.getSqlPara("erp.updateBarCode", param);
                Db.update(update);
            }
            redisTemplate.opsForValue().set("erpBarCode", JSON.toJSONString(erpBarCodeList));
        }

        // ERP需求单 ERP_ORDER_LIST
        SqlPara sqlParaOrderList = Db.getSqlPara("erp.selectOrderList");
        List<Record> recordOrderList = Db.find(sqlParaOrderList);
        List<ErpOrderList> erpOrderLists = RecordUtils.converModel(recordOrderList, ErpOrderList.class);
        //根据KEY 获取对应的 redis存储的value，存储数据为空进行新的存储
        String getErpOrderList = redisTemplate.opsForValue().get("erpOrderList");

        if (erpOrderLists != null && erpOrderLists.size() > 0 && StringUtils.isEmpty(getErpOrderList)) {
            for (ErpOrderList erpOrderList : erpOrderLists) {
                Kv param = Kv.create();
                param.put("orderNo", erpOrderList.getOrderNo());
                param.put("orderName", erpOrderList.getOrderName());
                SqlPara update = Db.getSqlPara("erp.updateOrderList", param);
                Db.update(update);
            }
            redisTemplate.opsForValue().set("erpOrderList", JSON.toJSONString(erpOrderLists));
        }
    }

    @JFinalTx
    @Override
    public String writeBack(List<ErpBarCodeList> erpBarCodeListList) {
        for (ErpBarCodeList erpBarCodeList : erpBarCodeListList) {
            Kv param = Kv.create();
            param.set("barcode",erpBarCodeList.getBarcode());
            param.set("orderName",erpBarCodeList.getOrderName());
            param.set("orderNo",erpBarCodeList.getOrderNo());
            param.set("qty",erpBarCodeList.getQty());
            param.set("orderType",erpBarCodeList.getOrder_type());
            param.set("inout",erpBarCodeList.getInOut());
            param.set("stockID",erpBarCodeList.getStockId());
            param.set("boxid",erpBarCodeList.getBoxNo());
            param.set("erpResult",erpBarCodeList.getErpResult());
            param.set("createUser", erpBarCodeList.getCreateUser());
            param.set("createDate",erpBarCodeList.getCreateDate());
            //param.set("wmsMark","N");
            SqlPara sqlPara = Db.getSqlPara("erp.insertBarcodeList",param);
            if (Db.update(sqlPara) == 0) {
                throw new RRException("writeBack失败！");
            }
        }
        return "200";
    }

    @Override
    public String updateErpStatus(ErpOrderList erpOrderList) {

        Kv param = Kv.create();
        param.put("orderState", erpOrderList.getOrderState());
        param.put("orderNo", erpOrderList.getOrderNo());
        param.put("orderName", erpOrderList.getOrderName());
        SqlPara update = Db.getSqlPara("erp.updateErpStatus", param);

        if(Db.update(update) == 0){
            return "500";
        }
        return "200";
    }


}
